# Operating systems

FEDORA = "fedora"
FEDORA_23 = "fedora-23"

DEBIAN = "debian"
DEBIAN_8 = "debian-8"

FEDORA_ALL = (FEDORA, FEDORA_23)
DEBIAN_ALL = (DEBIAN, DEBIAN_8)
OPERATING_SYSTEM_ALL = FEDORA_ALL + DEBIAN_ALL


# Firewall policy options

ALLOW = 'allow'
DENY = 'deny'
FIREWALL_POLICY_ALL = (ALLOW, DENY)


# VM label colors

RED = 'red'
ORANGE = 'orange'
YELLOW = 'yellow'
GREEN = 'green'
BLUE = 'blue'
PURPLE = 'purple'
BLACK = 'black'
GRAY = 'gray'
LABEL_COLOR_ALL = (RED, ORANGE, YELLOW, GREEN, BLUE, PURPLE, BLACK, GRAY)

